/*
 * Copyright (c) 2006-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: DSCUtil.java,v 1.19 2007/02/28 01:11:36 sshirasu Exp $
 */
package edu.indiana.extreme.xbaya.gpel;

import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import xsul5.wsdl.WsdlDefinitions;
import xsul5.wsdl.WsdlPortType;
import xsul5.wsdl.WsdlResolver;
import xsul5.wsdl.WsdlUtil;
import edu.indiana.extreme.xbaya.XBayaRuntimeException;
import edu.indiana.extreme.xbaya.gpel.script.BPELScript;
import edu.indiana.extreme.xbaya.graph.Graph;
import edu.indiana.extreme.xbaya.graph.util.GraphUtil;
import edu.indiana.extreme.xbaya.graph.ws.WSNode;
import edu.indiana.extreme.xbaya.util.WSDLUtil;
import edu.indiana.extreme.xbaya.wf.Workflow;

/**
 * @author Satoshi Shirasuna
 */
public class DSCUtil {

    /**
     * For debugging
     * 
     * @param args
     */
    public static void main(String[] args) {
        WsdlDefinitions awsdl = WsdlResolver.getInstance().loadWsdl(
                DSCUtil.class, "wsdls/math/adder-awsdl.xml");
        WsdlDefinitions cwsdl = convertToCWSDL(awsdl, URI
                .create("http://localhost"));
        System.out.println(cwsdl.xmlStringPretty());
    }

    /**
     * Creates CWSDLs for all WSDLs in a workflow.
     * 
     * @param workflow
     * @param dscURL
     * @return The Map<partnerLinkName, CWSDL>.
     * @throws URISyntaxException
     */
    public static Map<String, WsdlDefinitions> createCWSDLs(Workflow workflow,
            String dscURL) throws URISyntaxException {
        return createCWSDLs(workflow, new URI(dscURL));
    }

    /**
     * @param workflow
     * @param dscURL
     * @return The Map<partnerLinkName, CWSDL>.
     */
    public static Map<String, WsdlDefinitions> createCWSDLs(Workflow workflow,
            URI dscURL) {
        Map<String, WsdlDefinitions> WSDLMap = new HashMap<String, WsdlDefinitions>();
        Graph graph = workflow.getGraph();
        for (WSNode node : GraphUtil.getWSNodes(graph)) {
            String partnerLinkName = BPELScript.createPartnerLinkName(node
                    .getID());
            WsdlDefinitions wsdl = node.getComponent().getWSDL();
            if (WSDLUtil.isAWSDL(wsdl)) {
                wsdl = convertToCWSDL(WSDLUtil.deepClone(wsdl), dscURL);
            }
            WSDLMap.put(partnerLinkName, wsdl);
        }
        return WSDLMap;
    }

    /**
     * Converts a specified AWSDL to CWSDL using DSC URI.
     * 
     * @param definitions
     *            The specified AWSDL. This will be modified.
     * @param dscURI
     * @return The CWSDL converted.
     */
    public static WsdlDefinitions convertToCWSDL(WsdlDefinitions definitions,
            URI dscURI) {
        // Create a new List to avoid ConcurrentModificationException.
        List<WsdlPortType> portTypes = new ArrayList<WsdlPortType>();
        for (WsdlPortType portType : definitions.portTypes()) {
            portTypes.add(portType);
        }

        for (WsdlPortType portType : portTypes) {
            URI uri = creatEPR(dscURI, portType.getQName());
            WsdlUtil.createCWSDL(definitions, portType, uri);
        }
        return definitions;
    }

    private static URI creatEPR(URI dscURI, QName portTypeQName) {
        String encodedPortType;
        try {
            encodedPortType = URLEncoder.encode(portTypeQName.toString(),
                    "UTF-8");
            URI epr = dscURI.resolve("/" + encodedPortType);
            return epr;
        } catch (UnsupportedEncodingException e) {
            // Should not happen
            throw new XBayaRuntimeException(e);
        }
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2006-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
